// HTTPリクエスト
var Http = function(globalObj, loadingImagePath, timeoutMsec, suspendKeyAry, debugLvl){
	this.debugLvl = (typeof(debugLvl) == "undefined" ? 1 : debugLvl);

	// Error Check
	if(
			arguments.length	< 1
		||	typeof(globalObj)	== "undefined"
	){
		if(this.debugLvl >= 1){ alert(this.debugLvl, "Invalid parameter. => ["+ arguments.length + " : " + typeof(globalObj) + "]"); }
		throw -1;
	}

	this.timeoutMsec = (typeof(timeoutMsec) == "undefined" ? 10000 : timeoutMsec);
	this.globalObj = globalObj;
	this.gRequest = null;
	this.commFlag = false;
	this.funcLoadingIcon = new LoadingIcon(globalObj, loadingImagePath);
	this.errorFlag = false;

	// ユーザから通信を中断するためのキーハンドリング
	//   最低でも2種類のキーコントロールを設定することを推奨
	this.suspendKeyAry =	(typeof(suspendKeyAry) == "undefined"
								? 
									[
										 0x08	// BS or clearキー
										,0x1B	// ESCキー
									]
								:
									suspendKeyAry
							)
	;

	document.onkeydown = (function(){
		var this_ = arguments[0];

		return function(){
			try{
				if(this_.commFlag){
					for(var i=0; i<this_.suspendKeyAry.length; ++i){
						if(this_.suspendKeyAry[i] == event.keyCode){
							this_.globalObj.infoMsg(this_.debugLvl, "onkeydown event:" + event.keyCode);
							this_.abort();
							this_.onkeydown();
							break;
						}
					}
				}
			}catch(e){
			}
		};
	})(this);
}

Http.prototype = {
	// publicメソッド /////////////////////////////////////////////
	 start : function(method, url, headerTree, content){
		// Error Check
		if(
				arguments.length	< 2
			||	typeof(method)		== "undefined"
			||	typeof(url)			== "undefined"
		){
			this.globalObj.errorMsg(this.debugLvl, "Invalid parameter. => ["+ arguments.length + " : " + typeof(method) + " : " + typeof(url) + "]");
			throw -1;
		}
		if(this.commFlag){
			this.globalObj.errorMsg(1, "Http is already started.");
			throw -1;
		}
		this.commFlag = true;

		this.headerTree = headerTree;
		this.content = (typeof(content) == "undefined" ? null : content);
		this.gRequest = new XMLHttpRequest();

		// XMLHttpRequestオブジェクトの状態（readyState）が変化する度に呼び出されるイベントハンドラ
		this.gRequest.onreadystatechange = (function(){
			var this_ = arguments[0];

			return function(){
				this_._handleResult();
			};
		})(this);

		// 事前通信タイマー
		//   通信先のurlがnetaccess要素で通信制限されている場合、openメソッドで半永久的にブロッキングが発生
		//   これを防ぐための事前タイムアウトをここで仕込んでおく
		this._startTimer("pre timeout", true);

		this.gRequest.open(method, url, true/* 非同期 */);
	}

	//   イベントハンドラ   //
	//     必ずオーバーライドしてください。
	,done : function(xmlHttpRequest){}
	//     必要であれば外部からオーバーライドしてください。
	,onkeydown : function(){}
	,ontimeout : function(){}

	,isCommunicating : function(){
		return this.commFlag;
	}

	// 通信の中断 //
	,abort : function(preTimeoutFlag){
		if(!this.commFlag){
			this.globalObj.errorMsg(this.debugLvl, "Http is already stopped.");
			throw -1;
		}

		this._afterCommunication(preTimeoutFlag);

		this.globalObj.infoMsg(this.debugLvl, "abort");
		this.gRequest.abort();
		this.gRequest = null;
	}

	// privateメソッド //////////////////////////////////////////////
	,_handleResult : function(){
		try{
			// Auto Variables
			var headerStr = "";

			this.globalObj.infoMsg(this.debugLvl, "readyStateの値 : " + this.gRequest.readyState);

			switch(this.gRequest.readyState){
			case 1: // OPENED（open()メソッドは正常に実行された。HTTPリクエストヘッダーを設定するかsend()メソッド実行可能）
				clearTimeout(this.timerId);		// 事前通信タイマーを終了

				// リクエストヘッダの追加
				if(typeof(this.headerTree) != "undefined"){
					for(headerStr in this.headerTree){
						this.gRequest.setRequestHeader(headerStr, this.headerTree[headerStr]);
					}
				}

				this.funcLoadingIcon.start();	// 通信状態の表示

				this._startTimer("timeout", false);	// 通信タイマーを開始

				// リクエスト送信
				this.gRequest.send(this.content);
				break;
				
			case 4: // DONE（データ転送完了、もしくは転送中に何らかのエラー発生）
				this._afterCommunication();
				
				this.globalObj.infoMsg(this.debugLvl, "statusの値 : " + this.gRequest.status + " : " + this.gRequest.statusText);
				this.globalObj.infoMsg(this.debugLvl, this.gRequest.getAllResponseHeaders());

				this.done(this.gRequest);
				break;

			case 0: // UNSENT（オブジェクト構築済み）
			case 2: // HEADERS RECEIVED（HTTPリクエストヘッダーはサーバ側で受信済み）
			case 3: // LOADING（HTTPレスポンスボディを受信中）
			default:
				break;
			}
		}catch(e){
		}
	}

	,_afterCommunication : function(preTimeoutFlag){
		clearTimeout(this.timerId);		// タイマーを終了する
		if(true != preTimeoutFlag){
			this.funcLoadingIcon.stop();	// 通信状態の非表示
		}
		this.commFlag = false;
	}

	,_startTimer : function(timeoutMsg, preTimeoutFlag){
		// this.timeoutMsecに指定されたミリ秒を経過するとtimeout()を実行する
		this.timerId = setTimeout(
							 (function(){
								var this_ = arguments[0];

								return function(){
									// タイムアウト処理
									this_.globalObj.infoMsg(this_.debugLvl, timeoutMsg);
									this_.abort(preTimeoutFlag);
									this_.ontimeout();
								};
							 })(this)
							,this.timeoutMsec
		);
	}
}

